<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Processor\Quote;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Model\OrderInterface;
use Dedi\SyliusQuotePlugin\Model\OrderItemInterface;
use Sylius\Component\Core\Calculator\ProductVariantPricesCalculatorInterface;
use Sylius\Component\Core\Model\AdjustmentInterface;
use Sylius\Component\Order\Factory\AdjustmentFactoryInterface;
use Sylius\Component\Order\Model\OrderItemUnitInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

class QuotePromotionProcessor implements QuotePromotionProcessorInterface
{
    private AdjustmentFactoryInterface $adjustmentFactory;

    private TranslatorInterface $translator;

    private ProductVariantPricesCalculatorInterface $productVariantPricesCalculator;

    public function __construct(
        AdjustmentFactoryInterface $adjustmentFactory,
        TranslatorInterface $translator,
        ProductVariantPricesCalculatorInterface $productVariantPricesCalculator
    ) {
        $this->adjustmentFactory = $adjustmentFactory;
        $this->translator = $translator;
        $this->productVariantPricesCalculator = $productVariantPricesCalculator;
    }

    public function process(QuoteInterface $quote, OrderInterface $order): void
    {
        /** @var QuoteItemInterface $quoteItem */
        foreach ($quote->getItems() as $quoteItem) {
            $orderItem = $this->findOrderItemFromQuote($order, $quoteItem);
            if (null === $orderItem) {
                continue;
            }

            $this->removePromotions($orderItem);

            $originalPrice = $this->productVariantPricesCalculator->calculate($quoteItem->getVariant(), [
                'channel' => $order->getChannel(),
            ]);

            $this->addAdjustments($orderItem, $quoteItem, $originalPrice);
        }
    }

    private function findOrderItemFromQuote(OrderInterface $order, QuoteItemInterface $quoteItem): ?OrderItemInterface
    {
        /** @var OrderItemInterface|false $orderItem */
        $orderItem = $order->getItems()->filter(function (OrderItemInterface $orderItem) use ($quoteItem) {
            return $orderItem->getVariant() === $quoteItem->getVariant() && $orderItem->getQuoteItem() === $quoteItem;
        })->first();

        return false !== $orderItem ? $orderItem : null;
    }

    private function removePromotions(OrderItemInterface $orderItem): void
    {
        $adjustments = [
            AdjustmentInterface::ORDER_ITEM_PROMOTION_ADJUSTMENT,
            AdjustmentInterface::ORDER_UNIT_PROMOTION_ADJUSTMENT,
        ];

        foreach ($adjustments as $adjustmentType) {
            $orderItem->removeAdjustmentsRecursively($adjustmentType);
        }
    }

    private function addAdjustments(OrderItemInterface $item, QuoteItemInterface $quoteItem, int $originalPrice): void
    {
        /** @var OrderItemUnitInterface $unit */
        foreach ($item->getUnits() as $unit) {
            $unit->addAdjustment($this->adjustmentFactory->createWithData(
                AdjustmentInterface::ORDER_UNIT_PROMOTION_ADJUSTMENT,
                $this->translator->trans('sylius.ui.promotions'),
                ($quoteItem->getUnitPrice() - $originalPrice),
                false
            ));
        }
    }
}
